Package: rayimage
Type: Package
Title: Image Processing for Simulated Cameras
Version: 0.26.1
Authors@R: person("Tyler", "Morgan-Wall", email = "tylermw@gmail.com",
    role = c("aut", "cph", "cre"), comment = c(ORCID = "0000-0002-3131-3814"))
Maintainer: Tyler Morgan-Wall <tylermw@gmail.com>
Description: Uses convolution-based techniques to generate simulated camera bokeh, depth of field, and other camera effects, using an image and an optional depth map. Accepts both filename inputs and in-memory array representations of images and matrices, including common raster formats such as 'JPEG', 'PNG', 'TIFF', 'TGA', 'BMP', 'PSD', 'GIF', 'HDR', 'PIC', 'PNM', 'DNG', and 'EXR'. Includes functions to perform 2D convolutions, color correction, colorspace conversion, image/matrix reorientation and resizing, image and text overlays, exposure adjustment, camera vignette effects, and image titles.
License: GPL-3
LazyData: true
Depends: R (>= 4.3.0)
Imports: Rcpp, png, jpeg, grDevices, grid, tiff, systemfonts, ragg
Suggests: magick, testthat (>= 3.0.0), libopenexr (>= 3.4.12-4), cli
LinkingTo: Rcpp, RcppArmadillo, progress, tinydng, stbimageheaders
Encoding: UTF-8
URL: https://www.rayimage.dev,
        https://github.com/tylermorganwall/rayimage
BugReports: https://github.com/tylermorganwall/rayimage/issues
Config/testthat/edition: 3
Config/build/compilation-database: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-12 05:44:36 UTC; tyler
Author: Tyler Morgan-Wall [aut, cph, cre] (ORCID:
    <https://orcid.org/0000-0002-3131-3814>)
Repository: CRAN
Date/Publication: 2026-06-12 16:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-12 20:29:42 UTC; unix
Archs: rayimage.so.dSYM
