# Generated by roxygen2: do not edit by hand

S3method("[",rayimg)
S3method(print,rayimg)
export(CS_ACESCG)
export(CS_ADOBE)
export(CS_BT2020)
export(CS_P3D65)
export(CS_SRGB)
export(generate_2d_disk)
export(generate_2d_exponential)
export(generate_2d_gaussian)
export(get_string_dimensions)
export(interpolate_array)
export(plot_image)
export(plot_image_grid)
export(ray_read_image)
export(ray_write_image)
export(render_alpha_outline)
export(render_bokeh)
export(render_boolean_distance)
export(render_bw)
export(render_clamp)
export(render_color_correction)
export(render_convert_colorspace)
export(render_convolution)
export(render_convolution_fft)
export(render_crop)
export(render_exposure)
export(render_gamma_linear)
export(render_image_overlay)
export(render_padding)
export(render_reorient)
export(render_resized)
export(render_sprite_overlay)
export(render_stack)
export(render_text_image)
export(render_title)
export(render_to_display)
export(render_tonemap)
export(render_vibrance)
export(render_vignette)
export(render_white_balance)
import(grDevices)
import(grid)
import(png)
import(stats)
importFrom(Rcpp,evalCpp)
importFrom(ragg,agg_png)
useDynLib(rayimage, .registration = TRUE)
