# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

get_boolean_distance <- function(input) {
    .Call(`_rayimage_get_boolean_distance`, input)
}

encode_native_image_rcpp_1 <- function(bw_image) {
    .Call(`_rayimage_encode_native_image_rcpp_1`, bw_image)
}

encode_native_image_rcpp_3 <- function(r_image, g_image, b_image) {
    .Call(`_rayimage_encode_native_image_rcpp_3`, r_image, g_image, b_image)
}

encode_native_image_rcpp_4 <- function(r_image, g_image, b_image, a_image) {
    .Call(`_rayimage_encode_native_image_rcpp_4`, r_image, g_image, b_image, a_image)
}

read_dng_cpp <- function(filename, normalize = TRUE, pick = "largest") {
    .Call(`_rayimage_read_dng_cpp`, filename, normalize, pick)
}

dng_apply_active_area_cpp <- function(pixels, active_area) {
    .Call(`_rayimage_dng_apply_active_area_cpp`, pixels, active_area)
}

dng_exposure_normalize_cpp <- function(pixels, iso, exposure_time) {
    .Call(`_rayimage_dng_exposure_normalize_cpp`, pixels, iso, exposure_time)
}

dng_undo_baseline_exposure_cpp <- function(pixels, baseline_exposure) {
    .Call(`_rayimage_dng_undo_baseline_exposure_cpp`, pixels, baseline_exposure)
}

write_dng_cpp <- function(filename, pixels, meta, bitdepth = 16L, compress = TRUE) {
    .Call(`_rayimage_write_dng_cpp`, filename, pixels, meta, bitdepth, compress)
}

load_image_stb <- function(filename, width, height, channels, desired_channels) {
    .Call(`_rayimage_load_image_stb`, filename, width, height, channels, desired_channels)
}

subsample <- function(circle, size) {
    .Call(`_rayimage_subsample`, circle, size)
}

rayinterp2 <- function(image, XI, YI) {
    .Call(`_rayimage_rayinterp2`, image, XI, YI)
}

resize_image <- function(image, mag) {
    .Call(`_rayimage_resize_image`, image, mag)
}

resize_image_xy <- function(image, XI, YI) {
    .Call(`_rayimage_resize_image_xy`, image, XI, YI)
}

resize_matrix_stb <- function(image, width, height, method) {
    .Call(`_rayimage_resize_matrix_stb`, image, width, height, method)
}

generate_disk <- function(radius, dim, offsetx, offsety) {
    .Call(`_rayimage_generate_disk`, radius, dim, offsetx, offsety)
}

gen_ellipse <- function(intensity, width, height) {
    .Call(`_rayimage_gen_ellipse`, intensity, width, height)
}

subsample_rect <- function(rect, binsx, binsy) {
    .Call(`_rayimage_subsample_rect`, rect, binsx, binsy)
}

gen_circle_psf <- function(radius) {
    .Call(`_rayimage_gen_circle_psf`, radius)
}

is_inside <- function(sizehex, positionx, positiony, sinval, cosval) {
    .Call(`_rayimage_is_inside`, sizehex, positionx, positiony, sinval, cosval)
}

gen_hex_psf <- function(radius, rotation) {
    .Call(`_rayimage_gen_hex_psf`, radius, rotation)
}

psf <- function(image, blurmatrix, depthmap, depth, custombokeh, type, bokehintensity, bokehlimit, rotation, progbar, channel) {
    .Call(`_rayimage_psf`, image, blurmatrix, depthmap, depth, custombokeh, type, bokehintensity, bokehlimit, rotation, progbar, channel)
}

convolution_cpp <- function(image, kernel, progbar, channel, bloom_matrix) {
    .Call(`_rayimage_convolution_cpp`, image, kernel, progbar, channel, bloom_matrix)
}

