% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flipfunctions.R
\name{fliplr}
\alias{fliplr}
\title{Flip Left-Right}
\usage{
fliplr(x)
}
\arguments{
\item{x}{Matrix}
}
\value{
Flipped matrix
}
\description{
Flip Left-Right
}
\examples{
#Fake example
}
\keyword{internal}
