% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colorspace_helpers.R
\name{to_linear}
\alias{to_linear}
\title{sRGB to Linear}
\usage{
to_linear(x)
}
\arguments{
\item{x}{Values}
}
\value{
Transformed values
}
\description{
sRGB to Linear
}
\keyword{internal}
