% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_vignette_overlay.R
\name{make_vignette_overlay}
\alias{make_vignette_overlay}
\title{Make Vignette Overlay}
\usage{
make_vignette_overlay(
  width,
  height,
  intensity = 0.3,
  radius_multiplier = 1.3,
  radius = NULL,
  color = "#000000"
)
}
\arguments{
\item{width}{Width of the image.}

\item{height}{Height of the image.}

\item{intensity}{Default \code{0.4}. \code{1} is max intensity, \code{0} is min.}

\item{radius_multiplier}{Default \code{1.3}.}

\item{radius}{Default \code{NULL}. Max of height or width, divided by 2.}

\item{color}{Default \code{black}.}
}
\description{
Makes an overlay to simulate vignetting in a camera
}
\keyword{internal}
