% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colorspace_helpers.R
\name{to_srgb}
\alias{to_srgb}
\title{Linear to sRGB}
\usage{
to_srgb(x)
}
\arguments{
\item{x}{Values}
}
\value{
Transformed values
}
\description{
Linear to sRGB
}
\keyword{internal}
